/*
 * Dash Media Player: External Interface Gateway 
 * Copyright (c) 2008 TMT Digital LLC.
 *
 * Author:  Travis Tidwell | http://www.travistidwell.com
 *
 * Dependencies:
 *    jQuery Library
 *
 * This script is used as a connection interface gateway to control the player from an external source.
 * It is also used as a means for communication between multiple instances of the Dash Media Player
 * on the page.  This allows you to separate the Playlist and the actual
 * player on the page, but yet have them behave as a single player.  For more information on how you can
 * integrate this great feature on your website, visit http://www.tmtdigital.com/remoteplaylist
 *
 * License: GPL
 */

var dashReady = false;
var dashObjects = new Object;
var dashCallback = function( args ){};
var dashId = "dashplayer";

$(document).ready(function() {
   dashReady = true;  
});

function isDashReady() {
   return dashReady;
};

/**
 * Adds a Dash Player object to the list of avialable
 * players.
 */
function dashAddObject( id ) {
   dashObjects[id] = new Object;
   dashObjects[id].id = id;
   dashObjects[id].ready = false;
   dashObjects[id].object = null;
};

/**
 * Registers a dash player object.
 */
function dashRegisterObject( id ) {
   if( !dashObjects[id] ) {
      dashAddObject( id );
   }

   dashObjects[id].ready = true;
   
   startDash();      
};

/**
 * Checks to see if all the players have registered.
 */
 
function isDashRegistered() {
   var registered = true;
   for( id in dashObjects ) {
     registered &= dashObjects[id].ready;
   }
   return registered;
};

/** 
 * Starts the Dash Media Players
 */
function startDash()
{
   if( isDashRegistered() ) {
     for ( id in dashObjects ) {
         dashInitialize( id );
      }
   }
};

/**
 * Initializes the dash player object.
 */
 
function dashInitialize( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.initialize();
   }
};

/** 
 * Loads a single node into the player.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The node ID of the node you would like for this player to load.
 */
function dashLoadNode( id, nodeId ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.loadNode( nodeId );
       return true;
   }   
   return false;
};

/** 
 * Loads a media file in the player.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The file that you would like for the player to load. 
 */
function dashLoad( id, file ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.load( file );
       return true;
   }  
   return false;
};

/** 
 * Plays a media file.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The file that you would like for the player to play.  
 *          This is not necessary if you use dashLoadNode or dashLoad before making this call.
 *          If you call this function with the file provided, then the player will load that file
 *          before actually playing it.
 */
function dashPlay( id, file ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.play( file );
       return true;
   }   
   return false;
};

/** 
 * Pauses the media file that is playing in the player.
 *
 * @param - The ID of the player you would like to send this request too.
 */
function dashPause( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.pause();
       return true;
   }
   return false;
};

/** 
 * Stops the media file that is playing in the player.
 *
 * @param - The ID of the player you would like to send this request too.
 */
function dashStop( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.stop();
       return true;
   }
   return false;
};

/** 
 * Seeks the media file that is playing to the time specified.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The time that you would like to seek the track too.
 */
function dashSeek( id, seekTime ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.seek( seekTime );
       return true;
   }
   return false;
};

/** 
 * Sets the volume of the media being played.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The volume that you would like to set the media too.
 */
function dashVolume( id, vol ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.volume( vol );
       return true;
   }
   return false;
};

/** 
 * Gets the volume of the media being played.
 *
 * @param - The ID of the player you would like to send this request too.
 */
function dashGetVolume( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       return dashObj.getVolume();
   }
   return 0;
};

/** 
 * Sets the player into Full Screen mode..
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Boolean:  True - FullScreen, False - Normal
 */
function dashSetFullScreen( id, full ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.setFullScreen( full );
       return true;
   }
   return false;
};

/** 
 * Maximizes the player by getting rid of the playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Boolean:  True - Maximize, False - Minimize
 * @param - Boolean:  Used to indicate if you want the transition to be tweened.
 */
function dashSetMaximize( id, max, tween ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.setMaximize( max, tween );
       return true;
   }
   return false;
};

/** 
 * Maximizes the player by getting rid of the playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Boolean:  True - Show Menu, False - Hide Menu
 * @param - Boolean:  Used to indicate if you want the transition to be tweened.
 */
function dashSetMenu( id, menu, tween ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.setMenu( menu, tween );
       return true;
   }
   return false;
};

/** 
 * Sees if the player has already loaded a node.
 *
 * @param - The ID of the player you would like to send this request too.
 *
 * @return - Boolean (True if is has loaded content, False otherwise)
 */
function dashIsNodeLoaded( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {
      return (dashObj.isNodeLoaded());
   }
   return false;
};

/** 
 * Loads a playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The name of the playlist (view) you would like to load.
 */
function dashLoadPlaylist( id, playlist ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {      
       dashObj.loadPlaylist( playlist );
       return true;
   }
   return false;
};

/** 
 * Loads the previous item in the playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Indicate if you would like for the player to loop to the end of the list if it is already
 *          on the first item.
 * @param - Indicate if you would like for the playlist to play the file after it loads it.
 */
function dashLoadPrev( id, loop, playAfter ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {      
       dashObj.loadPrev( loop, playAfter );
       return true;
   }
   return false;
};

/** 
 * Loads the next item in the playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Indicate if you would like for the player to loop to the beginning of the list if it is already
 *          on the last item.
 * @param - Indicate if you would like for the playlist to play the file after it loads it.
 */
function dashLoadNext( id, loop, playAfter ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.loadNext( loop, playAfter );
       return true;
   }
   return false;
};

/** 
 * Loads the previous page in the playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Indicate if you would like for the player to loop to the last page if it is already
 *          on the first page.
 */
function dashPrevPage( id, loop ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {      
       dashObj.prevPage( loop );
       return true;
   }
   return false;
};

/** 
 * Loads the next page in the playlist.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - Indicate if you would like for the player to loop to the beginning page if it is already
 *          on the last page.
 */
function dashNextPage( id, loop ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.nextPage( loop );
       return true;
   }
   return false;
};

/** 
 * Sets the filter argument for the playlist currently loaded.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The argument that you would like to pass to your playlist to filter the content.
 * @param - The index of the argument.
 */
function dashSetFilter( id, argument, index ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setFilter( argument, index );
       return true;
   }
   return false;
};

function dashDebug( arg ) {
   //alert( arg.toString() );
   $('#dashdebug').append( arg + "<br/>" );
};

/** 
 * Resets the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 */
function dashResetControls( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.resetControls();
       return true;
   }
   return false;
};

/** 
 * Enables/Disables the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - true - Enable the controls :  false - Disable the controls.
 */
function dashEnableControls( id, enable ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.enableControls(enable);
       return true;
   }
   return false;
};

/** 
 * Sets the state of the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The state you would like to set this control bar too.
 *             "play"  - Play State
 *             "pause" - Pause State
 */
function dashSetControlState( id, state ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setControlState(state);
       return true;
   }
   return false;
};

/** 
 * Sets the total time print out of the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The time that you would like to set on the controls.
 */
function dashSetControlTime( id, time ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setControlTime(time);
       return true;
   }
   return false;
};

/** 
 * Sets the volume of the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The volume that you would like to set on the controls.
 */
function dashSetControlVolume( id, volume ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setControlVolume(volume);
       return true;
   }
   return false;
};

/** 
 * Sets the progress indication of the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The progress that you would like to set on the controls.
 */
function dashSetControlProgress( id, progress ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setControlProgress(progress);
       return true;
   }
   return false;
};

/** 
 * Sets the seek indication of the controls.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The seek that you would like to set on the controls.
 */
function dashSetControlSeek( id, seek ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setControlSeek(seek);
       return true;
   }
   return false;
};

/** 
 * Updates the controls given the play time and total time..
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The playhead time.
 * @param - The total time of the media being played.
 */
function dashControlUpdate( id, playTime, totalTime ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.controlUpdate(playTime, totalTime);
       return true;
   }
   return false;
};

/** 
 * Dynamically sets the Dash Media Player skin..
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The skin you would like to switch to.
 */
function dashSetSkin( id, skin ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.setSkin( skin );
       return true;
   }
   return false;
};

/** 
 * Calls a Dash Media Player service routine.  The player must have the external service
 * flag set to true in order for this to work properly.
 *
 * @param - The ID of the player you would like to send this request too.
 * @param - The command that you would like to send to the service.
 * @param - The callback that will get called with the response.
 */

function dashServiceCall() {
   var id = arguments[0];
   arguments.shift();
   var command = arguments[0];
   arguments.shift();
   dashCallback = arguments[0];
   arguments.shift();
   
   var dashObj = getDashObject( id );
   if( dashObj ) {         
       dashObj.serviceCall( command, arguments );
       return true;
   }
   return false;
};

/** 
 * Return from the service call.
 *
 * @param - An array of the arguments that gets passed back.
 */

function dashServiceReturn( args ) {
   dashCallback( args );   
};

/** 
 * YouTube wrapper functions.
 */
function onYouTubePlayerReady( playerId ) {
   dashId = playerId;
   var dashObj = getDashObject( playerId );
   if( dashObj ) {
      dashObj.addEventListener( "onStateChange", "youTubeOnStateChange" );
      dashObj.addEventListener( "onError", "youTubeOnError" );
      dashObj.onYouTubeReady();
   }
};

function youTubeOnStateChange( newState ) {
   var dashObj = getDashObject( dashId );
   if( dashObj ) {    
       dashObj.onYouTubeStateChange( newState );
   }     
};

function youTubeOnError( error ) {
   var dashObj = getDashObject( dashId );
   if( dashObj ) {    
       dashObj.onYouTubeError( error );
   }     
};

function youTubeLoad( id, youTubeId, startSeconds ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.loadVideoById( youTubeId, startSeconds );
   }
};

function youTubeCue( id, youTubeId, startSeconds ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.cueVideoById( youTubeId, startSeconds );
   }
};

function youTubeDestroy( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.destroy();
   }
};

function youTubeClear( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.clearVideo();
   }
};

function youTubeSetSize( id, _width, _height ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.setSize( _width, _height );
   }
};

function youTubePlay( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.playVideo();
   }
};

function youTubePause( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.pauseVideo();
   }
};

function youTubeStop( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.stopVideo();
   }
};

function youTubeSeek( id, seconds, allowSeekAhead ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.seekTo( seconds, allowSeekAhead );
   }
};

function youTubeGetBytesLoaded( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getVideoBytesLoaded();
   }
   return 0;
};

function youTubeGetBytesTotal( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getVideoBytesTotal();
   }
   return 0;
};

function youTubeGetCurrentTime( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getCurrentTime();
   }
   return 0;
};

function youTubeGetDuration( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getDuration();
   }
   return 0;
};

function youTubeSetVolume( id, newVolume ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
       dashObj.setVolume( newVolume );
   }
};

function youTubeGetVolume( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getVolume();
   }
   return 0;
};

function youTubeGetEmbedCode( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getEmbedCode();
   }
   return "";
};

function youTubeGetVideoUrl( id ) {
   var dashObj = getDashObject( id );
   if( dashObj ) {    
      return dashObj.getVideoUrl();
   }
   return "";
};

/** 
 * Gets the Dash Player Object given an ID
 *
 * @param - The ID of the player you would like to send this request too.
 */
function getDashObject( id ){
   var dashObj = null;       
                              
   // If there is a cached object, use it instead.
   if( dashObjects[id] ) {
      if( dashObjects[id].object ) {
         dashObj = dashObjects[id].object;
      }
   }
   
   if( !dashObj) {
      if( dashReady ) {   
         if(navigator.appName.indexOf("Microsoft") != -1) {
            dashObj = window[id];
         }
         else {
            if(document[id].length != undefined) {
               dashObj = document[id][1];
            }
            else {
               dashObj = document[id];
            }
         }
         
         if( dashObjects[id] ) {
            dashObjects[id].object = dashObj;
         }
      }
   }
   
   return dashObj;
};